package inapp_shop

import (
	"crypto/hmac"
	"crypto/sha256"
	"encoding/hex"
	"f5"
	"main/common"
	"main/constant"
	"main/model"
	"main/mt"
	"main/service"
	"main/vo"
	"q5"
	"sort"
	"strings"

	"github.com/gin-gonic/gin"
)

type InAppShopApi struct {
}

func (this *InAppShopApi) Purchase(c *gin.Context) {
	s := c.MustGet(constant.SESSION_KEY).(common.Session)
	if s == nil {
		return
	}
	reqJson := struct {
		GoodsId  int32 `json:"goods_id"`
		GoodsNum int32 `json:"goods_num"`
	}{}
	if err := c.ShouldBindJSON(&reqJson); err != nil {
		f5.RspErr(c, 401, "params parse error")
		return
	}
	goodsMeta := mt.Table.ShopGoods.GetById(int64(reqJson.GoodsId))
	if goodsMeta == nil {
		f5.RspErr(c, 401, "goodsId params error")
		return
	}
	if reqJson.GoodsNum != 1 {
		f5.RspErr(c, 401, "goodsNum params error")
		return
	}
	nowTime := int32(f5.GetApp().GetRealSeconds())
	orderModel := new(model.InAppOrder)
	orderModel.OrderId = q5.ToString(f5.GetApp().NewLockNodeUuid())
	orderModel.AccountId = s.GetAccountId()
	orderModel.GoodsId = reqJson.GoodsId
	orderModel.CreateTime = nowTime
	orderModel.ModifyTime = nowTime
	if err := orderModel.Create(); err != nil {
		f5.RspErr(c, 500, "server internal error")
		return
	}
	rspObj := struct {
		vo.BaseVo
		OrderId string `json:"order_id"`
	}{}
	rspObj.OrderId = orderModel.OrderId
	c.JSON(200, rspObj)
}

func (this *InAppShopApi) OrderInfo(c *gin.Context) {
	s := c.MustGet(constant.SESSION_KEY).(common.Session)
	if s == nil {
		return
	}
	orderId := c.Param("order_id")
	orderModel := new(model.InAppOrder)
	if err, found := orderModel.Find(s.GetAccountId(), orderId); err != nil {
		f5.RspErr(c, 500, "server internal error")
		return
	} else if !found {
		f5.RspErr(c, 1, "not found")
		return
	}
	rspObj := struct {
		vo.BaseVo
		Data struct {
			OrderId string `json:"order_id"`
			Status  int32  `json:"status"`
		} `json:"data"`
	}{}
	rspObj.Data.OrderId = orderModel.OrderId
	rspObj.Data.Status = orderModel.Status
	c.JSON(200, rspObj)
}

func (this *InAppShopApi) Callback(c *gin.Context) {
	reqObj := struct {
		Channel   string                   `json:"channel"`
		Records   []map[string]interface{} `json:"records"`
		Signature string                   `json:"sign"`
	}{}

	if err := c.ShouldBindJSON(&reqObj); err != nil || len(reqObj.Records) == 0 {
		f5.RspErr(c, 401, "invalid params")
		return
	}

	keys := []string{}
	rec := reqObj.Records[0]
	for k := range rec {
		keys = append(keys, k)
	}
	sort.Strings(keys)

	strb := strings.Builder{}
	strb.WriteString("channel=")
	strb.WriteString(reqObj.Channel)
	for _, k := range keys {
		strb.WriteByte('&')
		strb.WriteString(k)
		strb.WriteByte('=')
		strb.WriteString(q5.SafeToString(rec[k]))
	}

	secret := mt.Table.Config.GetShopSecret()
	mac := hmac.New(sha256.New, []byte(secret))
	_, _ = mac.Write([]byte(strb.String()))

	sign := strings.ToLower(hex.EncodeToString(mac.Sum(nil)))
	// f5.GetSysLog().Info("%s, %s, %s, %s", strb.String(), secret, sign, strings.ToLower(reqObj.Signature))
	if !strings.EqualFold(sign, strings.ToLower(reqObj.Signature)) {
		f5.RspErr(c, 401, "invalid sign")
		return
	}

	orderId := q5.SafeToString(rec["gameOrderId"])
	orderModel := new(model.InAppOrder)
	if err, found := orderModel.FindByOrderId(orderId); err != nil {
		f5.RspErr(c, 500, "server internal error")
		return
	} else if !found {
		f5.RspErr(c, 1, "order not found")
		return
	}

	status := q5.SafeToInt32(rec["status"])
	if status <= orderModel.Status {
		f5.RspErr(c, 401, "invalid status")
		return
	}

	orderModel.Status = status
	orderModel.SpOrderId = q5.SafeToString(rec["orderId"])
	orderModel.ModifyTime = int32(f5.GetApp().GetRealSeconds())
	if orderModel.UpdateFields([]string{"status", "sp_order_id", "modifytime"}) != nil {
		f5.RspErr(c, 500, "server internal error")
		return
	}

	f5.RspErr(c, 0, "")

	ordergoods := mt.Table.ShopGoods.GetById(int64(orderModel.GoodsId))
	if ordergoods == nil {
		f5.GetSysLog().Error("fail dispatch goods:%s", orderId)
		return
	}

	itemcfg := mt.Table.Item.GetById(int64(ordergoods.GetItemId()))
	if itemcfg == nil {
		f5.GetSysLog().Error("item error, fail dispatch goods:%s, %d", orderId, ordergoods.GetItemId())
		return
	}

	user := new(model.User)
	if err, found := user.Find(orderModel.AccountId); err != nil {
		f5.GetSysLog().Error("internal error, fail dispatch goods:%s", orderId)
		return
	} else if !found {
		f5.GetSysLog().Error("not found user, fail dispatch goods:%s, %s", orderId, orderModel.AccountId)
		return
	}
	
	award := new(vo.Award)
	award.AddItem(ordergoods.GetItemId(), ordergoods.GetAmount())	

	service.Award.HandleAwards(user, award, constant.ITEM_OP_SOURCE_INAPP_SHOP, nil, nil)

	service.Task.UpdateTaskProgress(orderModel.AccountId, constant.TASK_COND_BACK_BUY, itemcfg.GetItemType(), ordergoods.GetAmount(), nil)
}
